-- Model: CA - Conway's Game of Life
--
-- Author: Tiago Carneiro 
--
-- Lesson: TerraME models MUST have the following structure
--             1) Define space
--				2) Define behavior
--				3) Define time
--				4) Run simulations 
--

--------------------------------------------------------------
-- GLOBALS
--------------------------------------------------------------

DEAD = 1
ALIVE = 2

-- Fun seeds: 
--SEED =  1 -- walking pattern
--SEED = 2 -- big monster on top
--SEED = 9	-- small monster on left top
--SEED = 10 -- 2 crosses
--SEED = 12 -- lots of flowers exploding and a nice dynamic pattern last forever
--SEED = 28 --a  new dynamic pattern appears on left bottom and is destroyed near to the end
--SEED = 31  -- new dynamic pattern on the right bottom
--rand = Random{ seed = SEED }

rand = Random()

--------------------------------------------------------------
-- MODEL (1. space, 2. behavior, and 3. time)
--------------------------------------------------------------

-------------
--# SPACE #
-------------
world = CellularSpace{ 
	xdim = 50
}
forEachCell(world, function(cell) 
	cell.state = rand:integer(DEAD,ALIVE)
	--print(cell.id, cell.state)
end)

world:createNeighborhood{
		strategy = "moore",
		self = false
}

----------------
--# BEHAVIOR #
----------------

-- Auxiliary funtion to count the number of ALIVE neighbors of a cell
function count_neighbors(cs)
	
	local count = 0
	
	forEachNeighbor(cs, function(cell, neighbor)
		if neighbor.past.state == ALIVE then
			count = count + 1
		end
	end)
	
	return count

end

--Model rules
function changes ()

	forEachCell(world, function(cell)
		neighbors = count_neighbors(cell)
		if neighbors < 2 then
			cell.state = DEAD
		elseif neighbors > 3 then
			cell.state = DEAD
		elseif neighbors == 3 and cell.past.state == DEAD then
			cell.state = ALIVE
		end
	end)
	
end

-----------
--# TIME #
-----------

t = Timer{
	Event{action = function()
		world:synchronize()
		changes( )
		world:notify()
	end}
}


--------------------------------------------------------------
-- SIMULATION
--------------------------------------------------------------

leg = Legend {
	colorBar = {
		{value = DEAD, color = "white"},
		{value = ALIVE, color = "black"}
	}
}

Observer { subject  = world,  type = "map" ,	attributes  = {"state"}, 	legends = {leg}}

world:notify()  -- needed just if you want to see the initial state (before simulation runs)

t:execute(1000)



